# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: ibc/core/client/v1/client.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from gogoproto import gogo_pb2 as gogoproto_dot_gogo__pb2
from google.protobuf import any_pb2 as google_dot_protobuf_dot_any__pb2
from cosmos.upgrade.v1beta1 import upgrade_pb2 as cosmos_dot_upgrade_dot_v1beta1_dot_upgrade__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='ibc/core/client/v1/client.proto',
  package='ibc.core.client.v1',
  syntax='proto3',
  serialized_options=b'Z8github.com/cosmos/ibc-go/v2/modules/core/02-client/types',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x1fibc/core/client/v1/client.proto\x12\x12ibc.core.client.v1\x1a\x14gogoproto/gogo.proto\x1a\x19google/protobuf/any.proto\x1a$cosmos/upgrade/v1beta1/upgrade.proto\"\x85\x01\n\x15IdentifiedClientState\x12\'\n\tclient_id\x18\x01 \x01(\tB\x14\xf2\xde\x1f\x10yaml:\"client_id\"\x12\x43\n\x0c\x63lient_state\x18\x02 \x01(\x0b\x32\x14.google.protobuf.AnyB\x17\xf2\xde\x1f\x13yaml:\"client_state\"\"\x96\x01\n\x18\x43onsensusStateWithHeight\x12\x30\n\x06height\x18\x01 \x01(\x0b\x32\x1a.ibc.core.client.v1.HeightB\x04\xc8\xde\x1f\x00\x12H\n\x0f\x63onsensus_state\x18\x02 \x01(\x0b\x32\x14.google.protobuf.AnyB\x19\xf2\xde\x1f\x15yaml\"consensus_state\"\"\xa9\x01\n\x15\x43lientConsensusStates\x12\'\n\tclient_id\x18\x01 \x01(\tB\x14\xf2\xde\x1f\x10yaml:\"client_id\"\x12g\n\x10\x63onsensus_states\x18\x02 \x03(\x0b\x32,.ibc.core.client.v1.ConsensusStateWithHeightB\x1f\xf2\xde\x1f\x17yaml:\"consensus_states\"\xc8\xde\x1f\x00\"\xb8\x01\n\x14\x43lientUpdateProposal\x12\r\n\x05title\x18\x01 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x02 \x01(\t\x12\x37\n\x11subject_client_id\x18\x03 \x01(\tB\x1c\xf2\xde\x1f\x18yaml:\"subject_client_id\"\x12=\n\x14substitute_client_id\x18\x04 \x01(\tB\x1f\xf2\xde\x1f\x1byaml:\"substitute_client_id\":\x04\x88\xa0\x1f\x00\"\xcc\x01\n\x0fUpgradeProposal\x12\r\n\x05title\x18\x01 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x02 \x01(\t\x12\x30\n\x04plan\x18\x03 \x01(\x0b\x32\x1c.cosmos.upgrade.v1beta1.PlanB\x04\xc8\xde\x1f\x00\x12U\n\x15upgraded_client_state\x18\x04 \x01(\x0b\x32\x14.google.protobuf.AnyB \xf2\xde\x1f\x1cyaml:\"upgraded_client_state\":\x0c\x88\xa0\x1f\x00\x98\xa0\x1f\x00\xe8\xa0\x1f\x01\"|\n\x06Height\x12\x33\n\x0frevision_number\x18\x01 \x01(\x04\x42\x1a\xf2\xde\x1f\x16yaml:\"revision_number\"\x12\x33\n\x0frevision_height\x18\x02 \x01(\x04\x42\x1a\xf2\xde\x1f\x16yaml:\"revision_height\":\x08\x88\xa0\x1f\x00\x98\xa0\x1f\x00\"=\n\x06Params\x12\x33\n\x0f\x61llowed_clients\x18\x01 \x03(\tB\x1a\xf2\xde\x1f\x16yaml:\"allowed_clients\"B:Z8github.com/cosmos/ibc-go/v2/modules/core/02-client/typesb\x06proto3'
  ,
  dependencies=[gogoproto_dot_gogo__pb2.DESCRIPTOR,google_dot_protobuf_dot_any__pb2.DESCRIPTOR,cosmos_dot_upgrade_dot_v1beta1_dot_upgrade__pb2.DESCRIPTOR,])




_IDENTIFIEDCLIENTSTATE = _descriptor.Descriptor(
  name='IdentifiedClientState',
  full_name='ibc.core.client.v1.IdentifiedClientState',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='client_id', full_name='ibc.core.client.v1.IdentifiedClientState.client_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\362\336\037\020yaml:\"client_id\"', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='client_state', full_name='ibc.core.client.v1.IdentifiedClientState.client_state', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\362\336\037\023yaml:\"client_state\"', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=143,
  serialized_end=276,
)


_CONSENSUSSTATEWITHHEIGHT = _descriptor.Descriptor(
  name='ConsensusStateWithHeight',
  full_name='ibc.core.client.v1.ConsensusStateWithHeight',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='height', full_name='ibc.core.client.v1.ConsensusStateWithHeight.height', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\310\336\037\000', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='consensus_state', full_name='ibc.core.client.v1.ConsensusStateWithHeight.consensus_state', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\362\336\037\025yaml\"consensus_state\"', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=279,
  serialized_end=429,
)


_CLIENTCONSENSUSSTATES = _descriptor.Descriptor(
  name='ClientConsensusStates',
  full_name='ibc.core.client.v1.ClientConsensusStates',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='client_id', full_name='ibc.core.client.v1.ClientConsensusStates.client_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\362\336\037\020yaml:\"client_id\"', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='consensus_states', full_name='ibc.core.client.v1.ClientConsensusStates.consensus_states', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\362\336\037\027yaml:\"consensus_states\"\310\336\037\000', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=432,
  serialized_end=601,
)


_CLIENTUPDATEPROPOSAL = _descriptor.Descriptor(
  name='ClientUpdateProposal',
  full_name='ibc.core.client.v1.ClientUpdateProposal',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='title', full_name='ibc.core.client.v1.ClientUpdateProposal.title', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='description', full_name='ibc.core.client.v1.ClientUpdateProposal.description', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='subject_client_id', full_name='ibc.core.client.v1.ClientUpdateProposal.subject_client_id', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\362\336\037\030yaml:\"subject_client_id\"', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='substitute_client_id', full_name='ibc.core.client.v1.ClientUpdateProposal.substitute_client_id', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\362\336\037\033yaml:\"substitute_client_id\"', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=b'\210\240\037\000',
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=604,
  serialized_end=788,
)


_UPGRADEPROPOSAL = _descriptor.Descriptor(
  name='UpgradeProposal',
  full_name='ibc.core.client.v1.UpgradeProposal',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='title', full_name='ibc.core.client.v1.UpgradeProposal.title', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='description', full_name='ibc.core.client.v1.UpgradeProposal.description', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='plan', full_name='ibc.core.client.v1.UpgradeProposal.plan', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\310\336\037\000', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='upgraded_client_state', full_name='ibc.core.client.v1.UpgradeProposal.upgraded_client_state', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\362\336\037\034yaml:\"upgraded_client_state\"', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=b'\210\240\037\000\230\240\037\000\350\240\037\001',
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=791,
  serialized_end=995,
)


_HEIGHT = _descriptor.Descriptor(
  name='Height',
  full_name='ibc.core.client.v1.Height',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='revision_number', full_name='ibc.core.client.v1.Height.revision_number', index=0,
      number=1, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\362\336\037\026yaml:\"revision_number\"', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='revision_height', full_name='ibc.core.client.v1.Height.revision_height', index=1,
      number=2, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\362\336\037\026yaml:\"revision_height\"', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=b'\210\240\037\000\230\240\037\000',
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=997,
  serialized_end=1121,
)


_PARAMS = _descriptor.Descriptor(
  name='Params',
  full_name='ibc.core.client.v1.Params',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='allowed_clients', full_name='ibc.core.client.v1.Params.allowed_clients', index=0,
      number=1, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\362\336\037\026yaml:\"allowed_clients\"', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1123,
  serialized_end=1184,
)

_IDENTIFIEDCLIENTSTATE.fields_by_name['client_state'].message_type = google_dot_protobuf_dot_any__pb2._ANY
_CONSENSUSSTATEWITHHEIGHT.fields_by_name['height'].message_type = _HEIGHT
_CONSENSUSSTATEWITHHEIGHT.fields_by_name['consensus_state'].message_type = google_dot_protobuf_dot_any__pb2._ANY
_CLIENTCONSENSUSSTATES.fields_by_name['consensus_states'].message_type = _CONSENSUSSTATEWITHHEIGHT
_UPGRADEPROPOSAL.fields_by_name['plan'].message_type = cosmos_dot_upgrade_dot_v1beta1_dot_upgrade__pb2._PLAN
_UPGRADEPROPOSAL.fields_by_name['upgraded_client_state'].message_type = google_dot_protobuf_dot_any__pb2._ANY
DESCRIPTOR.message_types_by_name['IdentifiedClientState'] = _IDENTIFIEDCLIENTSTATE
DESCRIPTOR.message_types_by_name['ConsensusStateWithHeight'] = _CONSENSUSSTATEWITHHEIGHT
DESCRIPTOR.message_types_by_name['ClientConsensusStates'] = _CLIENTCONSENSUSSTATES
DESCRIPTOR.message_types_by_name['ClientUpdateProposal'] = _CLIENTUPDATEPROPOSAL
DESCRIPTOR.message_types_by_name['UpgradeProposal'] = _UPGRADEPROPOSAL
DESCRIPTOR.message_types_by_name['Height'] = _HEIGHT
DESCRIPTOR.message_types_by_name['Params'] = _PARAMS
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

IdentifiedClientState = _reflection.GeneratedProtocolMessageType('IdentifiedClientState', (_message.Message,), {
  'DESCRIPTOR' : _IDENTIFIEDCLIENTSTATE,
  '__module__' : 'ibc.core.client.v1.client_pb2'
  # @@protoc_insertion_point(class_scope:ibc.core.client.v1.IdentifiedClientState)
  })
_sym_db.RegisterMessage(IdentifiedClientState)

ConsensusStateWithHeight = _reflection.GeneratedProtocolMessageType('ConsensusStateWithHeight', (_message.Message,), {
  'DESCRIPTOR' : _CONSENSUSSTATEWITHHEIGHT,
  '__module__' : 'ibc.core.client.v1.client_pb2'
  # @@protoc_insertion_point(class_scope:ibc.core.client.v1.ConsensusStateWithHeight)
  })
_sym_db.RegisterMessage(ConsensusStateWithHeight)

ClientConsensusStates = _reflection.GeneratedProtocolMessageType('ClientConsensusStates', (_message.Message,), {
  'DESCRIPTOR' : _CLIENTCONSENSUSSTATES,
  '__module__' : 'ibc.core.client.v1.client_pb2'
  # @@protoc_insertion_point(class_scope:ibc.core.client.v1.ClientConsensusStates)
  })
_sym_db.RegisterMessage(ClientConsensusStates)

ClientUpdateProposal = _reflection.GeneratedProtocolMessageType('ClientUpdateProposal', (_message.Message,), {
  'DESCRIPTOR' : _CLIENTUPDATEPROPOSAL,
  '__module__' : 'ibc.core.client.v1.client_pb2'
  # @@protoc_insertion_point(class_scope:ibc.core.client.v1.ClientUpdateProposal)
  })
_sym_db.RegisterMessage(ClientUpdateProposal)

UpgradeProposal = _reflection.GeneratedProtocolMessageType('UpgradeProposal', (_message.Message,), {
  'DESCRIPTOR' : _UPGRADEPROPOSAL,
  '__module__' : 'ibc.core.client.v1.client_pb2'
  # @@protoc_insertion_point(class_scope:ibc.core.client.v1.UpgradeProposal)
  })
_sym_db.RegisterMessage(UpgradeProposal)

Height = _reflection.GeneratedProtocolMessageType('Height', (_message.Message,), {
  'DESCRIPTOR' : _HEIGHT,
  '__module__' : 'ibc.core.client.v1.client_pb2'
  # @@protoc_insertion_point(class_scope:ibc.core.client.v1.Height)
  })
_sym_db.RegisterMessage(Height)

Params = _reflection.GeneratedProtocolMessageType('Params', (_message.Message,), {
  'DESCRIPTOR' : _PARAMS,
  '__module__' : 'ibc.core.client.v1.client_pb2'
  # @@protoc_insertion_point(class_scope:ibc.core.client.v1.Params)
  })
_sym_db.RegisterMessage(Params)


DESCRIPTOR._options = None
_IDENTIFIEDCLIENTSTATE.fields_by_name['client_id']._options = None
_IDENTIFIEDCLIENTSTATE.fields_by_name['client_state']._options = None
_CONSENSUSSTATEWITHHEIGHT.fields_by_name['height']._options = None
_CONSENSUSSTATEWITHHEIGHT.fields_by_name['consensus_state']._options = None
_CLIENTCONSENSUSSTATES.fields_by_name['client_id']._options = None
_CLIENTCONSENSUSSTATES.fields_by_name['consensus_states']._options = None
_CLIENTUPDATEPROPOSAL.fields_by_name['subject_client_id']._options = None
_CLIENTUPDATEPROPOSAL.fields_by_name['substitute_client_id']._options = None
_CLIENTUPDATEPROPOSAL._options = None
_UPGRADEPROPOSAL.fields_by_name['plan']._options = None
_UPGRADEPROPOSAL.fields_by_name['upgraded_client_state']._options = None
_UPGRADEPROPOSAL._options = None
_HEIGHT.fields_by_name['revision_number']._options = None
_HEIGHT.fields_by_name['revision_height']._options = None
_HEIGHT._options = None
_PARAMS.fields_by_name['allowed_clients']._options = None
# @@protoc_insertion_point(module_scope)



# # -*- coding: utf-8 -*-
# # ------------------------------------------------------------------------------
# #
# #   Copyright 2018-2021 Fetch.AI Limited
# #
# #   Licensed under the Apache License, Version 2.0 (the "License");
# #   you may not use this file except in compliance with the License.
# #   You may obtain a copy of the License at
# #
# #       http://www.apache.org/licenses/LICENSE-2.0
# #
# #   Unless required by applicable law or agreed to in writing, software
# #   distributed under the License is distributed on an "AS IS" BASIS,
# #   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# #   See the License for the specific language governing permissions and
# #   limitations under the License.
# #
# # ------------------------------------------------------------------------------
#
# """Tests for the CosmosLedger client module of the Clients Package."""
#
# import base64
# import gzip
# import json
# import os
# import tempfile
# import unittest
# from typing import Optional
# from unittest.mock import patch
#
# from google.protobuf.json_format import MessageToDict, ParseDict
#
# from cosmpy.auth.interface import Auth
# from cosmpy.auth.rest_client import AuthRestClient
# from cosmpy.bank.rest_client import BankRestClient
# from cosmpy.clients.crypto import CosmosCrypto
# from cosmpy.cosmwasm.rest_client import CosmWasmRestClient
# from cosmpy.crypto.address import Address
# from cosmpy.crypto.keypairs import PrivateKey
# from cosmpy.protos.cosmos.auth.v1beta1.auth_pb2 import BaseAccount
# from cosmpy.protos.cosmos.auth.v1beta1.query_pb2 import (
#     QueryAccountRequest,
#     QueryAccountResponse,
#     QueryParamsRequest,
#     QueryParamsResponse,
# )
# from cosmpy.protos.cosmos.base.abci.v1beta1.abci_pb2 import TxResponse
# from cosmpy.protos.cosmos.base.v1beta1.coin_pb2 import Coin
# from cosmpy.protos.cosmos.tx.v1beta1.service_pb2 import (
#     BroadcastTxRequest,
#     BroadcastTxResponse,
#     GetTxRequest,
#     GetTxResponse,
#     GetTxsEventRequest,
#     GetTxsEventResponse,
#     SimulateRequest,
#     SimulateResponse,
# )
# from cosmpy.protos.cosmos.tx.v1beta1.tx_pb2 import Tx
# from cosmpy.tx.interface import TxInterface
# from tests.helpers import MockRestClient
#
# # Private key
# PRIVATE_KEY = PrivateKey(
#     bytes.fromhex("deadbeefdeadbeefdeadbeefdeadbeefdeadbeefdeadbeefdeadbeefdeadbeef")
# )
#
# # Addresses
# ADDRESS_PK = Address(PRIVATE_KEY)
# ADDRESS_OTHER = "fetchaddressaddressaddressaddressaddressaddr"
#
# # Public keys
# PUBLIC_KEY_PK_BASE64 = base64.b64encode(PRIVATE_KEY.public_key_bytes).decode()
#
# # Node config
# CHAIN_ID = "testing"
#
# # Auth data
# ACCOUNT_NUMBER = 0
# SEQUENCE = 1
#
# # Tx params
# GAS_LIMIT = 987654321
# LABEL = "label"
#
# # Coins
# DENOM = "stake"
# AMOUNT = "1234"
# COINS = [Coin(amount=AMOUNT, denom=DENOM)]
#
# # CosmWasm
# WASM_MSG = {"key": "value"}
# WASM_MSG_BASE64 = base64.b64encode(json.dumps(WASM_MSG).encode("UTF8")).decode()
# CODE_ID = 42
# CONTRACT_ADDRESS = "fetchcontractcontractcontractcontractcontrac"
# CONTRACT_FILENAME = "dummy_contract.wasm"
# CONTRACT_BYTECODE = b"ABCD"
#
#
# class MockAuth(Auth):
#     """Mock Auth client"""
#
#     def __init__(
#         self,
#         address: Address,
#         account_number: int,
#         sequence: int,
#     ):
#         """
#         Create mock Auth client
#
#         :param address: Mock address to be returned in response
#         :param account_number: Mock account_number to be returned in response
#         :param sequence: Mock sequence to be returned in response
#         """
#         self.address = address
#         self.account_number = account_number
#         self.sequence = sequence
#
#     def Account(self, request: QueryAccountRequest) -> QueryAccountResponse:
#         """Queries account data - sequence, account_id, etc."""
#         content = {
#             "account": {
#                 "@type": "/cosmos.auth.v1beta1.BaseAccount",
#                 "address": str(self.address),
#                 "pub_key": {},
#                 "account_number": str(self.account_number),
#                 "sequence": str(self.sequence),
#             }
#         }
#         return ParseDict(content, QueryAccountResponse())
#
#     def Params(self, request: QueryParamsRequest) -> QueryParamsResponse:
#         """Queries all parameters"""
#         raise NotImplementedError("Method not implemented!")
#
#
# class MockTx(TxInterface):
#     """Mock Tx client"""
#
#     def __init__(self, response_code: int):
#         """
#         Create mock Tx client
#
#         :param response_code: Code in GetTxResponse returned on GetTx call
#         """
#         self.response_code = response_code
#         self.last_broadcast_tx_request: Optional[BroadcastTxRequest] = None
#
#     def Simulate(self, request: SimulateRequest) -> SimulateResponse:
#         """Simulate executing a transaction to estimate gas usage."""
#         raise NotImplementedError("Method not implemented!")
#
#     def GetTx(self, request: GetTxRequest) -> GetTxResponse:
#         """GetTx fetches a tx by hash."""
#         return GetTxResponse()
#
#     def BroadcastTx(self, request: BroadcastTxRequest) -> BroadcastTxResponse:
#         """BroadcastTx broadcast transaction."""
#         self.last_broadcast_tx_request = request
#         return BroadcastTxResponse(tx_response=TxResponse(code=self.response_code))
#
#     def GetTxsEvent(self, request: GetTxsEventRequest) -> GetTxsEventResponse:
#         """GetTxsEvent fetches txs by event."""
#         raise NotImplementedError("Method not implemented!")
#
#
# def mock_get_code_id(response: GetTxResponse) -> int:
#     """Get code id from store code transaction response"""
#     assert isinstance(response, GetTxResponse)
#     return CODE_ID
#
#
# def mock_get_contract_address(response: GetTxResponse) -> str:
#     """Get code id from store code transaction response"""
#     assert isinstance(response, GetTxResponse)
#     return CONTRACT_ADDRESS
#
#
# class CosmosLedgerTestCase(unittest.TestCase):
#     """Test case of CosmosLedger module."""
#
#     @classmethod
#     def setUpClass(cls):
#         """Set up test case."""
#
#         # Mock Auth response for __init__
#         mock_auth = MockAuth(ADDRESS_PK, ACCOUNT_NUMBER, SEQUENCE)
#         content = MessageToDict(
#             mock_auth.Account(QueryAccountRequest(address=str(ADDRESS_PK)))
#         )
#
#         mock_rest_client = MockRestClient(json.dumps(content))
#         cls.crypto = CosmosCrypto(
#             private_key=PRIVATE_KEY, account_number=ACCOUNT_NUMBER
#         )
#         cls.ledger = CosmosLedger(
#             rest_node_address="some_rest_node_address", chain_id=CHAIN_ID
#         )
#         cls.ledger.auth_client = mock_auth
#         cls.ledger.rest_client = mock_rest_client
#
#     def test_init(self):
#         """Test correct initialisation."""
#         assert str(ADDRESS_PK) == str(self.crypto.get_address())
#         assert ACCOUNT_NUMBER == self.crypto.account_number
#         assert CHAIN_ID == self.ledger.chain_id
#         assert PRIVATE_KEY.public_key_bytes == self.crypto.get_pubkey_as_bytes()
#         assert PRIVATE_KEY == self.crypto.private_key
#
#     def test_get_packed_send_msg(self):
#         """Test correct generation of packed send msg."""
#         expected_result = {
#             "@type": "/cosmos.bank.v1beta1.MsgSend",
#             "fromAddress": str(ADDRESS_PK),
#             "toAddress": str(ADDRESS_OTHER),
#             "amount": [{"denom": DENOM, "amount": AMOUNT}],
#         }
#
#         msg = self.ledger.get_packed_send_msg(ADDRESS_PK, ADDRESS_OTHER, COINS)
#         assert MessageToDict(msg) == expected_result
#
#     def test_get_packed_init_msg(self):
#         """Test correct generation of packed instantiate msg."""
#         expected_result = {
#             "@type": "/cosmwasm.wasm.v1.MsgInstantiateContract",
#             "sender": str(ADDRESS_PK),
#             "codeId": str(CODE_ID),
#             "label": LABEL,
#             "msg": WASM_MSG_BASE64,
#             "funds": [{"denom": DENOM, "amount": AMOUNT}],
#         }
#
#         msg = self.ledger.get_packed_init_msg(
#             ADDRESS_PK, CODE_ID, WASM_MSG, LABEL, COINS
#         )
#         assert MessageToDict(msg) == expected_result
#
#     def test_get_packed_exec_msg(self):
#         """Test correct generation of packed execute msg."""
#         expected_result = {
#             "@type": "/cosmwasm.wasm.v1.MsgExecuteContract",
#             "sender": str(ADDRESS_PK),
#             "contract": str(CONTRACT_ADDRESS),
#             "msg": WASM_MSG_BASE64,
#             "funds": [{"denom": DENOM, "amount": AMOUNT}],
#         }
#
#         msg = self.ledger.get_packed_exec_msg(
#             ADDRESS_PK, CONTRACT_ADDRESS, WASM_MSG, COINS
#         )
#         assert MessageToDict(msg) == expected_result
#
#     def test_get_packed_store_msg(self):
#         """Test correct generation of packed store msg."""
#         with tempfile.NamedTemporaryFile(suffix=CONTRACT_FILENAME, delete=False) as tmp:
#             tmp.write(CONTRACT_BYTECODE)
#             tmp.flush()
#         msg = self.ledger.get_packed_store_msg(ADDRESS_PK, tmp.name)
#         os.unlink(tmp.name)
#
#         msg_dict = MessageToDict(msg)
#         assert len(msg_dict) == 3
#         assert msg_dict["@type"] == "/cosmwasm.wasm.v1.MsgStoreCode"
#         assert msg_dict["sender"] == str(ADDRESS_PK)
#         zipped_bytecode: bytes = base64.b64decode(msg_dict["wasmByteCode"])
#         original_bytecode: bytes = gzip.decompress(zipped_bytecode)
#         self.assertEqual(original_bytecode, CONTRACT_BYTECODE)
#
#     def test_generate_tx(self):
#         """Test correct generation of Tx."""
#         expected_result = {
#             "body": {
#                 "messages": [
#                     {
#                         "@type": "/cosmos.bank.v1beta1.MsgSend",
#                         "fromAddress": str(ADDRESS_PK),
#                         "toAddress": str(ADDRESS_OTHER),
#                         "amount": [{"denom": DENOM, "amount": AMOUNT}],
#                     }
#                 ],
#                 "memo": LABEL,
#             },
#             "authInfo": {
#                 "signerInfos": [
#                     {
#                         "publicKey": {
#                             "@type": "/cosmos.crypto.secp256k1.PubKey",
#                             "key": PUBLIC_KEY_PK_BASE64,
#                         },
#                         "modeInfo": {"single": {"mode": "SIGN_MODE_DIRECT"}},
#                         "sequence": str(SEQUENCE),
#                     }
#                 ],
#                 "fee": {
#                     "amount": [{"denom": DENOM, "amount": AMOUNT}],
#                     "gasLimit": str(GAS_LIMIT),
#                 },
#             },
#         }
#
#         msg = self.ledger.get_packed_send_msg(ADDRESS_PK, ADDRESS_OTHER, COINS)
#         tx = self.ledger.generate_tx(
#             [msg], [ADDRESS_PK], [PRIVATE_KEY.public_key_bytes], COINS, LABEL, GAS_LIMIT
#         )
#
#         assert MessageToDict(tx) == expected_result
#
#     def test_sign_tx(self):
#         """Test correct generation of Tx."""
#         expected_result = {
#             "body": {
#                 "messages": [
#                     {
#                         "@type": "/cosmos.bank.v1beta1.MsgSend",
#                         "fromAddress": str(ADDRESS_PK),
#                         "toAddress": str(ADDRESS_OTHER),
#                         "amount": [{"denom": DENOM, "amount": AMOUNT}],
#                     }
#                 ],
#                 "memo": LABEL,
#             },
#             "authInfo": {
#                 "signerInfos": [
#                     {
#                         "publicKey": {
#                             "@type": "/cosmos.crypto.secp256k1.PubKey",
#                             "key": PUBLIC_KEY_PK_BASE64,
#                         },
#                         "modeInfo": {"single": {"mode": "SIGN_MODE_DIRECT"}},
#                         "sequence": str(SEQUENCE),
#                     }
#                 ],
#                 "fee": {
#                     "amount": [{"denom": DENOM, "amount": AMOUNT}],
#                     "gasLimit": str(GAS_LIMIT),
#                 },
#             },
#             "signatures": [
#                 "N/S3QvMc3zvS9X3UW20hPLxbXW4zE+x1M60xKsf5jE9FO6Q7vcVgEVIOj43bGZxh2rAfXcwM5u+hC7qRxOSgtA=="
#             ],
#         }
#
#         # Generate and sign transaction
#         msg = self.ledger.get_packed_send_msg(ADDRESS_PK, ADDRESS_OTHER, COINS)
#         tx = self.ledger.generate_tx(
#             [msg], [ADDRESS_PK], [PRIVATE_KEY.public_key_bytes], COINS, LABEL, GAS_LIMIT
#         )
#         self.ledger.sign_tx(self.crypto, tx)
#         dict_tx = MessageToDict(tx)
#
#         # Check if signature has correct length
#         assert len(base64.b64decode(dict_tx["signatures"][0])) == 64
#         # Replace with returned signature because signature is non-deterministic
#         expected_result["signatures"] = dict_tx["signatures"]
#
#         assert dict_tx == expected_result
#
#     def test_broadcast_tx_success(self):
#         """Test broadcast Tx with positive result."""
#         tx = self.ledger.generate_tx([], [], [], COINS, LABEL, GAS_LIMIT)
#
#         mock_tx_client = MockTx(response_code=0)
#         self.ledger.tx_client = mock_tx_client
#
#         result = self.ledger.broadcast_tx(tx, 1)
#         self.assertIsInstance(result, GetTxResponse)
#
#     def test_broadcast_tx_fail(self):
#         """Test broadcast Tx with negative result."""
#         tx = self.ledger.generate_tx([], [], [], COINS, LABEL, GAS_LIMIT)
#
#         # Response code different from 0 means Tx error
#         mock_tx_client = MockTx(response_code=1)
#         self.ledger.tx_client = mock_tx_client
#
#         # Check if broadcasting fails
#         self.assertRaises(BroadcastException, self.ledger.broadcast_tx, tx, 0)
#
#     def test_send_tokens(self):
#         """Test send tokens method with positive result."""
#
#         mock_tx_client = MockTx(response_code=0)
#         self.ledger.tx_client = mock_tx_client
#
#         result = self.ledger.send_funds(self.crypto, ADDRESS_OTHER, COINS)
#         self.assertIsInstance(result, dict)
#
#         # Reconstruct original Tx from last tx request bytes
#         tx = Tx()
#         tx.ParseFromString(mock_tx_client.last_broadcast_tx_request.tx_bytes)
#
#         assert len(tx.body.messages) == 1
#         assert tx.body.messages[0].type_url == "/cosmos.bank.v1beta1.MsgSend"
#
#     def test_deploy_contract(self):
#         """Test deploy contract method with positive result."""
#
#         mock_tx_client = MockTx(response_code=0)
#         self.ledger.tx_client = mock_tx_client
#
#         with patch.object(self.ledger, "get_code_id", mock_get_code_id):
#             with tempfile.NamedTemporaryFile(
#                 suffix=CONTRACT_FILENAME, delete=False
#             ) as tmp:
#                 tmp.write(CONTRACT_BYTECODE)
#                 tmp.flush()
#             code_id, _ = self.ledger.deploy_contract(self.crypto, tmp.name)
#             os.unlink(tmp.name)
#
#         assert code_id == CODE_ID
#
#         # Reconstruct original Tx from last tx request bytes
#         tx = Tx()
#         tx.ParseFromString(mock_tx_client.last_broadcast_tx_request.tx_bytes)
#
#         assert len(tx.body.messages) == 1
#         assert tx.body.messages[0].type_url == "/cosmwasm.wasm.v1.MsgStoreCode"
#
#     def test_init_contract(self):
#         """Test init contract method with positive result."""
#
#         mock_tx_client = MockTx(response_code=0)
#         self.ledger.tx_client = mock_tx_client
#
#         with patch.object(
#             self.ledger, "get_contract_address", mock_get_contract_address
#         ):
#             contract_address, _ = self.ledger.instantiate_contract(
#                 self.crypto, CODE_ID, WASM_MSG, LABEL
#             )
#         assert contract_address == CONTRACT_ADDRESS
#
#         # Reconstruct original Tx from last tx request bytes
#         tx = Tx()
#         tx.ParseFromString(mock_tx_client.last_broadcast_tx_request.tx_bytes)
#
#         assert len(tx.body.messages) == 1
#         assert (
#             tx.body.messages[0].type_url == "/cosmwasm.wasm.v1.MsgInstantiateContract"
#         )
#
#     def test_execute_contract(self):
#         """Test execute contract method with positive result."""
#
#         mock_tx_client = MockTx(response_code=0)
#         self.ledger.tx_client = mock_tx_client
#
#         result = self.ledger.execute_contract(self.crypto, CONTRACT_ADDRESS, WASM_MSG)
#         assert result == ({}, 0)
#
#         # Reconstruct original Tx from last tx request bytes
#         tx = Tx()
#         tx.ParseFromString(mock_tx_client.last_broadcast_tx_request.tx_bytes)
#
#         assert len(tx.body.messages) == 1
#         assert tx.body.messages[0].type_url == "/cosmwasm.wasm.v1.MsgExecuteContract"
#
#     def test_get_code_id(self):
#         """Test get code id from response with positive result."""
#
#         raw_log_dict = [
#             {
#                 "events": [
#                     {},
#                     {
#                         "type": "message",
#                         "attributes": [
#                             {"key": "code_id", "value": str(CODE_ID)},
#                         ],
#                     },
#                 ]
#             }
#         ]
#
#         tx_response = GetTxResponse()
#         tx_response.tx_response.raw_log = json.dumps(raw_log_dict)
#
#         result = self.ledger.get_code_id(tx_response)
#         assert result == CODE_ID
#
#     def test_get_contract_address(self):
#         """Test get contract address from response with positive result."""
#
#         raw_log_dict = [
#             {
#                 "events": [
#                     {
#                         "type": "wasm",
#                         "attributes": [
#                             {"key": "_contract_address", "value": CONTRACT_ADDRESS}
#                         ],
#                     },
#                 ]
#             }
#         ]
#
#         tx_response = GetTxResponse()
#         tx_response.tx_response.raw_log = json.dumps(raw_log_dict)
#
#         result = self.ledger.get_contract_address(tx_response)
#         assert result == CONTRACT_ADDRESS
#
#     def test_get_balance(self):
#         """Test get balance for the positive result."""
#
#         content = {"balance": {"denom": "stake", "amount": "1234"}}
#
#         mock_rest_client = MockRestClient(json.dumps(content))
#         self.ledger.bank_client = BankRestClient(mock_rest_client)
#         response = self.ledger.get_balance("address", "stake")
#
#         assert response == 1234
#         assert (
#             mock_rest_client.last_base_url == "/cosmos/bank/v1beta1/balances/address/"
#         )
#
#     def test_get_balances(self):
#         """Test get balances for the positive result."""
#
#         content = {
#             "balances": [{"denom": "stake", "amount": "1234"}],
#             "pagination": {"next_key": None, "total": 0},
#         }
#
#         mock_rest_client = MockRestClient(json.dumps(content))
#         self.ledger.bank_client = BankRestClient(mock_rest_client)
#         response = self.ledger.get_balances("address")
#
#         assert str(response) == '[denom: "stake"\namount: "1234"\n]'
#         assert mock_rest_client.last_base_url == "/cosmos/bank/v1beta1/balances/address"
#
#     def test_query_account_data(self):
#         """Test query account data for the positive result."""
#
#         content = {
#             "account": {
#                 "@type": "/cosmos.auth.v1beta1.BaseAccount",
#                 "address": "fetch1h6974x4dspft29r9gyegtajyzaht2cdh0rt93w",
#                 "pub_key": {
#                     "@type": "/cosmos.crypto.secp256k1.PubKey",
#                     "key": "A2BjpEo54gBpulf9CrA+6tGBASFC8okaO1DYTimk/Jwp",
#                 },
#                 "account_number": "0",
#                 "sequence": "1",
#             }
#         }
#         account_response = ParseDict(content, QueryAccountResponse())
#
#         account = BaseAccount()
#         if account_response.account.Is(BaseAccount.DESCRIPTOR):
#             account_response.account.Unpack(account)
#
#         mock_rest_client = MockRestClient(json.dumps(content))
#         self.ledger.auth_client = AuthRestClient(mock_rest_client)
#         response = self.ledger.query_account_data("address")
#
#         assert response == account
#         assert mock_rest_client.last_base_url == "/cosmos/auth/v1beta1/accounts/address"
#
#     def test_query_contract_state(self):
#         """Test query contract state for the positive result."""
#
#         raw_content = b'{"data": {"balance":"1"}}'
#         expected_response = {"balance": "1"}
#
#         mock_rest_client = MockRestClient(raw_content)
#         self.ledger.wasm_client = CosmWasmRestClient(mock_rest_client)
#         response = self.ledger.query_contract_state("fetchcontractaddress", {})
#
#         assert response == expected_response
#         assert (
#             mock_rest_client.last_base_url
#             == "/wasm/v1/contract/fetchcontractaddress/smart/e30="
#         )



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2018-2021 Fetch.AI Limited
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------

import json
from datetime import datetime
from typing import Any, Optional

from cosmpy.aerial.client import LedgerClient, prepare_and_broadcast_basic_transaction
from cosmpy.aerial.contract.cosmwasm import (
    create_cosmwasm_execute_msg,
    create_cosmwasm_instantiate_msg,
    create_cosmwasm_store_code_msg,
)
from cosmpy.aerial.tx import Transaction
from cosmpy.aerial.tx_helpers import SubmittedTx
from cosmpy.aerial.wallet import Wallet
from cosmpy.crypto.address import Address
from cosmpy.crypto.hashfuncs import sha256
from cosmpy.protos.cosmos.base.query.v1beta1.pagination_pb2 import PageRequest
from cosmpy.protos.cosmwasm.wasm.v1.query_pb2 import (
    QueryCodesRequest,
    QuerySmartContractStateRequest,
)


def _compute_digest(path: str) -> bytes:
    with open(path, "rb") as input_file:
        return sha256(input_file.read())


def _generate_label(digest: bytes) -> str:
    now = datetime.utcnow()
    return f"{digest.hex()[:14]}-{now.strftime('%Y%m%d%H%M%S')}"


class LedgerContract:
    def __init__(
        self,
        path: Optional[str],
        client: LedgerClient,
        address: Optional[Address] = None,
        digest: Optional[bytes] = None,
    ):
        self._path = path
        self._client = client
        self._address = address

        # select the digest either by computing it from the provided contract or by the value specified by
        # the user
        self._digest: Optional[bytes] = digest
        if path is not None:
            self._digest = _compute_digest(str(self._path))

        # attempt to lookup the code id from the network by digest
        if self._digest is not None:
            self._code_id = self._find_contract_id_by_digest(self._digest)
        else:
            self._code_id = None

    @property
    def path(self) -> Optional[str]:
        return self._path

    @property
    def digest(self) -> Optional[bytes]:
        return self._digest

    @property
    def code_id(self) -> Optional[int]:
        return self._code_id

    @property
    def address(self) -> Optional[Address]:
        return self._address

    def store(
        self,
        sender: Wallet,
        gas_limit: Optional[int] = None,
        memo: Optional[str] = None,
    ) -> int:
        if self._path is None:
            raise RuntimeError("Unable to upload code, no contract provided")

        # build up the store transaction
        tx = Transaction()
        tx.add_message(create_cosmwasm_store_code_msg(self._path, sender.address()))

        submitted_tx = prepare_and_broadcast_basic_transaction(
            self._client, tx, sender, gas_limit=gas_limit, memo=memo
        ).wait_to_complete()

        # extract the code id
        self._code_id = submitted_tx.contract_code_id
        if self._code_id is None:
            raise RuntimeError("Unable to extract contract code id")

        return self._code_id

    def instantiate(
        self,
        code_id: int,
        args: Any,
        sender: Wallet,
        label: Optional[str] = None,
        gas_limit: Optional[int] = None,
        admin_address: Optional[Address] = None,
        funds: Optional[str] = None,
    ) -> Address:

        assert self._digest is not None

        label = label or _generate_label(bytes(self._digest))

        # build up the store transaction
        tx = Transaction()
        tx.add_message(
            create_cosmwasm_instantiate_msg(
                code_id,
                args,
                label,
                sender.address(),
                admin_address=admin_address,
                funds=funds,
            )
        )

        submitted_tx = prepare_and_broadcast_basic_transaction(
            self._client, tx, sender, gas_limit=gas_limit
        ).wait_to_complete()

        # store the contract address
        self._address = submitted_tx.contract_address
        if self._address is None:
            raise RuntimeError("Unable to extract contract code id")

        return self._address

    def deploy(
        self,
        args: Any,
        sender: Wallet,
        label: Optional[str] = None,
        store_gas_limit: Optional[int] = None,
        instantiate_gas_limit: Optional[int] = None,
        admin_address: Optional[Address] = None,
        funds: Optional[str] = None,
    ) -> Address:

        # in the case where the contract is already deployed
        if self._address is not None and self._code_id is not None:
            return self._address

        assert self._address is None

        if self._code_id is None:
            self.store(sender, gas_limit=store_gas_limit)

        assert self._code_id is not None

        return self.instantiate(
            self._code_id,
            args,
            sender,
            label=label,
            gas_limit=instantiate_gas_limit,
            admin_address=admin_address,
            funds=funds,
        )

    def execute(
        self,
        args: Any,
        sender: Wallet,
        gas_limit: Optional[int] = None,
        funds: Optional[str] = None,
    ) -> SubmittedTx:
        if self._address is None:
            raise RuntimeError("Contract appears not to be deployed currently")

        # build up the store transaction
        tx = Transaction()
        tx.add_message(
            create_cosmwasm_execute_msg(
                sender.address(), self._address, args, funds=funds
            )
        )

        return prepare_and_broadcast_basic_transaction(
            self._client, tx, sender, gas_limit=gas_limit
        )

    def query(self, args: Any) -> Any:
        if self._address is None:
            raise RuntimeError("Contract appears not to be deployed currently")

        req = QuerySmartContractStateRequest(
            address=str(self._address), query_data=json.dumps(args).encode("UTF8")
        )
        resp = self._client.wasm.SmartContractState(req)
        return json.loads(resp.data)

    def _find_contract_id_by_digest(self, digest: bytes) -> Optional[int]:
        code_id = None

        pagination = None
        while True:
            req = QueryCodesRequest(pagination=pagination)
            resp = self._client.wasm.Codes(req)

            for code_info in resp.code_infos:
                if code_info.data_hash == digest:
                    code_id = int(code_info.code_id)
                    break

            # exit the search loop if we have successfully found our code id
            if code_id is not None:
                break

            # exit the search loop when we can't iterate any further
            if len(resp.pagination.next_key) == 0:
                break

            # proceed to the next page
            pagination = PageRequest(key=resp.pagination.next_key)

        return code_id



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2018-2022 Fetch.AI Limited
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------
"""Tests for REST implementation of IBC Core Client."""
import json
from typing import Dict, Tuple
from unittest import TestCase

from google.protobuf.json_format import ParseDict
from google.protobuf.wrappers_pb2 import Int32Value  # noqa # needed for protobuf decode

from cosmpy.ibc.core.client.rest_client import IBCCoreClientRestClient  # type: ignore
from cosmpy.protos.ibc.core.client.v1.query_pb2 import (
    QueryClientParamsRequest,
    QueryClientParamsResponse,
    QueryClientStateRequest,
    QueryClientStateResponse,
    QueryClientStatesRequest,
    QueryClientStatesResponse,
    QueryConsensusStateRequest,
    QueryConsensusStateResponse,
    QueryConsensusStatesRequest,
    QueryConsensusStatesResponse,
)
from tests.helpers import MockRestClient

TYPE = {
    "@type": "type.googleapis.com/google.protobuf.Int32Value",
    "value": "42",
}


class IBCCoreClientRestClientTestCase(TestCase):
    """Test case for IBCCoreClientRestClient class."""

    REST_CLIENT = IBCCoreClientRestClient

    def make_clients(
        self, response_content: Dict
    ) -> Tuple[MockRestClient, IBCCoreClientRestClient]:
        """
        Make  mock client and rest client api for specific content.

        :param response_content: dict
        :return: rest client instance
        """
        mock_client = MockRestClient(json.dumps(response_content).encode("utf-8"))
        rest_client = self.REST_CLIENT(mock_client)
        return mock_client, rest_client

    def test_ClientState(self):
        """Test ClientState method."""
        content = {
            "client_state": TYPE,
            "proof": "string",
            "proof_height": {"revision_number": "1", "revision_height": "1"},
        }
        mock_client, rest_client = self.make_clients(content)
        expected_response = ParseDict(content, QueryClientStateResponse())

        assert (
            rest_client.ClientState(QueryClientStateRequest(client_id="client_id"))
            == expected_response
        )
        assert (
            mock_client.last_base_url
            == "/ibc/core/client/v1beta1/client_states/client_id"
        )

    def test_ClientStates(self):
        """Test ClientStates method."""
        content = {
            "client_states": [{"client_id": "string", "client_state": TYPE}],
            "pagination": {"next_key": "string", "total": "1"},
        }
        mock_client, rest_client = self.make_clients(content)
        expected_response = ParseDict(content, QueryClientStatesResponse())

        assert rest_client.ClientStates(QueryClientStatesRequest()) == expected_response
        assert mock_client.last_base_url == "/ibc/core/client/v1beta1/client_states"

    def test_ConsensusState(self):
        """Test ConsensusState method."""
        content = {
            "consensus_state": TYPE,
            "proof": "string",
            "proof_height": {"revision_number": "1", "revision_height": "1"},
        }
        mock_client, rest_client = self.make_clients(content)
        expected_response = ParseDict(content, QueryConsensusStateResponse())

        assert (
            rest_client.ConsensusState(
                QueryConsensusStateRequest(
                    revision_number=1, client_id="client_id", revision_height=1
                )
            )
            == expected_response
        )
        assert (
            mock_client.last_base_url
            == "/ibc/core/client/v1beta1/consensus_states/client_id/revision/1/height/1"
        )

    def test_ConsensusStates(self):
        """Test ConsensusStates method."""
        content = {
            "consensus_states": [
                {
                    "height": {
                        "revision_number": "1",
                        "revision_height": "1",
                    },
                    "consensus_state": TYPE,
                }
            ],
            "pagination": {"next_key": "string", "total": "1"},
        }
        mock_client, rest_client = self.make_clients(content)
        expected_response = ParseDict(content, QueryConsensusStatesResponse())

        assert (
            rest_client.ConsensusStates(
                QueryConsensusStatesRequest(client_id="client_id")
            )
            == expected_response
        )
        assert (
            mock_client.last_base_url
            == "/ibc/core/client/v1beta1/consensus_states/client_id"
        )

    def test_ClientParams(self):
        """Test ClientParams method."""
        content = {}
        mock_client, rest_client = self.make_clients(content)
        expected_response = ParseDict(content, QueryClientParamsResponse())

        assert rest_client.ClientParams(QueryClientParamsRequest()) == expected_response
        assert mock_client.last_base_url == "/ibc/client/v1beta1/params"



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2018-2022 Fetch.AI Limited
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------
"""Interface for the IBC Applications Transfer functionality of CosmosSDK."""

from abc import ABC, abstractmethod

from cosmpy.protos.ibc.applications.transfer.v1.query_pb2 import (
    QueryDenomTraceRequest,
    QueryDenomTraceResponse,
    QueryDenomTracesRequest,
    QueryDenomTracesResponse,
    QueryParamsRequest,
    QueryParamsResponse,
)


class IBCApplicationsTransfer(ABC):
    """IBC Applications Transfer abstract class."""

    @abstractmethod
    def DenomTrace(self, request: QueryDenomTraceRequest) -> QueryDenomTraceResponse:
        """
        DenomTrace queries a denomination trace information.

        :param request: QueryDenomTraceRequest
        :return: QueryDenomTraceResponse
        """

    @abstractmethod
    def DenomTraces(self, request: QueryDenomTracesRequest) -> QueryDenomTracesResponse:
        """
        DenomTraces queries all denomination traces.

        :param request: QueryDenomTracesRequest
        :return: QueryDenomTracesResponse
        """

    @abstractmethod
    def Params(self, request: QueryParamsRequest) -> QueryParamsResponse:
        """
        Params queries all parameters of the ibc-transfer module.

        :param request: QueryParamsRequest
        :return: QueryParamsResponse
        """



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: cosmos/mint/v1beta1/query.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from gogoproto import gogo_pb2 as gogoproto_dot_gogo__pb2
from google.api import annotations_pb2 as google_dot_api_dot_annotations__pb2
from cosmos.mint.v1beta1 import mint_pb2 as cosmos_dot_mint_dot_v1beta1_dot_mint__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='cosmos/mint/v1beta1/query.proto',
  package='cosmos.mint.v1beta1',
  syntax='proto3',
  serialized_options=b'Z)github.com/cosmos/cosmos-sdk/x/mint/types',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x1f\x63osmos/mint/v1beta1/query.proto\x12\x13\x63osmos.mint.v1beta1\x1a\x14gogoproto/gogo.proto\x1a\x1cgoogle/api/annotations.proto\x1a\x1e\x63osmos/mint/v1beta1/mint.proto\"\x14\n\x12QueryParamsRequest\"H\n\x13QueryParamsResponse\x12\x31\n\x06params\x18\x01 \x01(\x0b\x32\x1b.cosmos.mint.v1beta1.ParamsB\x04\xc8\xde\x1f\x00\"\x17\n\x15QueryInflationRequest\"[\n\x16QueryInflationResponse\x12\x41\n\tinflation\x18\x01 \x01(\x0c\x42.\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\xc8\xde\x1f\x00\"\x1e\n\x1cQueryAnnualProvisionsRequest\"j\n\x1dQueryAnnualProvisionsResponse\x12I\n\x11\x61nnual_provisions\x18\x01 \x01(\x0c\x42.\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\xc8\xde\x1f\x00\x32\xc5\x03\n\x05Query\x12\x80\x01\n\x06Params\x12\'.cosmos.mint.v1beta1.QueryParamsRequest\x1a(.cosmos.mint.v1beta1.QueryParamsResponse\"#\x82\xd3\xe4\x93\x02\x1d\x12\x1b/cosmos/mint/v1beta1/params\x12\x8c\x01\n\tInflation\x12*.cosmos.mint.v1beta1.QueryInflationRequest\x1a+.cosmos.mint.v1beta1.QueryInflationResponse\"&\x82\xd3\xe4\x93\x02 \x12\x1e/cosmos/mint/v1beta1/inflation\x12\xa9\x01\n\x10\x41nnualProvisions\x12\x31.cosmos.mint.v1beta1.QueryAnnualProvisionsRequest\x1a\x32.cosmos.mint.v1beta1.QueryAnnualProvisionsResponse\".\x82\xd3\xe4\x93\x02(\x12&/cosmos/mint/v1beta1/annual_provisionsB+Z)github.com/cosmos/cosmos-sdk/x/mint/typesb\x06proto3'
  ,
  dependencies=[gogoproto_dot_gogo__pb2.DESCRIPTOR,google_dot_api_dot_annotations__pb2.DESCRIPTOR,cosmos_dot_mint_dot_v1beta1_dot_mint__pb2.DESCRIPTOR,])




_QUERYPARAMSREQUEST = _descriptor.Descriptor(
  name='QueryParamsRequest',
  full_name='cosmos.mint.v1beta1.QueryParamsRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=140,
  serialized_end=160,
)


_QUERYPARAMSRESPONSE = _descriptor.Descriptor(
  name='QueryParamsResponse',
  full_name='cosmos.mint.v1beta1.QueryParamsResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='params', full_name='cosmos.mint.v1beta1.QueryParamsResponse.params', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\310\336\037\000', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=162,
  serialized_end=234,
)


_QUERYINFLATIONREQUEST = _descriptor.Descriptor(
  name='QueryInflationRequest',
  full_name='cosmos.mint.v1beta1.QueryInflationRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=236,
  serialized_end=259,
)


_QUERYINFLATIONRESPONSE = _descriptor.Descriptor(
  name='QueryInflationResponse',
  full_name='cosmos.mint.v1beta1.QueryInflationResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='inflation', full_name='cosmos.mint.v1beta1.QueryInflationResponse.inflation', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec\310\336\037\000', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=261,
  serialized_end=352,
)


_QUERYANNUALPROVISIONSREQUEST = _descriptor.Descriptor(
  name='QueryAnnualProvisionsRequest',
  full_name='cosmos.mint.v1beta1.QueryAnnualProvisionsRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=354,
  serialized_end=384,
)


_QUERYANNUALPROVISIONSRESPONSE = _descriptor.Descriptor(
  name='QueryAnnualProvisionsResponse',
  full_name='cosmos.mint.v1beta1.QueryAnnualProvisionsResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='annual_provisions', full_name='cosmos.mint.v1beta1.QueryAnnualProvisionsResponse.annual_provisions', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec\310\336\037\000', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=386,
  serialized_end=492,
)

_QUERYPARAMSRESPONSE.fields_by_name['params'].message_type = cosmos_dot_mint_dot_v1beta1_dot_mint__pb2._PARAMS
DESCRIPTOR.message_types_by_name['QueryParamsRequest'] = _QUERYPARAMSREQUEST
DESCRIPTOR.message_types_by_name['QueryParamsResponse'] = _QUERYPARAMSRESPONSE
DESCRIPTOR.message_types_by_name['QueryInflationRequest'] = _QUERYINFLATIONREQUEST
DESCRIPTOR.message_types_by_name['QueryInflationResponse'] = _QUERYINFLATIONRESPONSE
DESCRIPTOR.message_types_by_name['QueryAnnualProvisionsRequest'] = _QUERYANNUALPROVISIONSREQUEST
DESCRIPTOR.message_types_by_name['QueryAnnualProvisionsResponse'] = _QUERYANNUALPROVISIONSRESPONSE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

QueryParamsRequest = _reflection.GeneratedProtocolMessageType('QueryParamsRequest', (_message.Message,), {
  'DESCRIPTOR' : _QUERYPARAMSREQUEST,
  '__module__' : 'cosmos.mint.v1beta1.query_pb2'
  # @@protoc_insertion_point(class_scope:cosmos.mint.v1beta1.QueryParamsRequest)
  })
_sym_db.RegisterMessage(QueryParamsRequest)

QueryParamsResponse = _reflection.GeneratedProtocolMessageType('QueryParamsResponse', (_message.Message,), {
  'DESCRIPTOR' : _QUERYPARAMSRESPONSE,
  '__module__' : 'cosmos.mint.v1beta1.query_pb2'
  # @@protoc_insertion_point(class_scope:cosmos.mint.v1beta1.QueryParamsResponse)
  })
_sym_db.RegisterMessage(QueryParamsResponse)

QueryInflationRequest = _reflection.GeneratedProtocolMessageType('QueryInflationRequest', (_message.Message,), {
  'DESCRIPTOR' : _QUERYINFLATIONREQUEST,
  '__module__' : 'cosmos.mint.v1beta1.query_pb2'
  # @@protoc_insertion_point(class_scope:cosmos.mint.v1beta1.QueryInflationRequest)
  })
_sym_db.RegisterMessage(QueryInflationRequest)

QueryInflationResponse = _reflection.GeneratedProtocolMessageType('QueryInflationResponse', (_message.Message,), {
  'DESCRIPTOR' : _QUERYINFLATIONRESPONSE,
  '__module__' : 'cosmos.mint.v1beta1.query_pb2'
  # @@protoc_insertion_point(class_scope:cosmos.mint.v1beta1.QueryInflationResponse)
  })
_sym_db.RegisterMessage(QueryInflationResponse)

QueryAnnualProvisionsRequest = _reflection.GeneratedProtocolMessageType('QueryAnnualProvisionsRequest', (_message.Message,), {
  'DESCRIPTOR' : _QUERYANNUALPROVISIONSREQUEST,
  '__module__' : 'cosmos.mint.v1beta1.query_pb2'
  # @@protoc_insertion_point(class_scope:cosmos.mint.v1beta1.QueryAnnualProvisionsRequest)
  })
_sym_db.RegisterMessage(QueryAnnualProvisionsRequest)

QueryAnnualProvisionsResponse = _reflection.GeneratedProtocolMessageType('QueryAnnualProvisionsResponse', (_message.Message,), {
  'DESCRIPTOR' : _QUERYANNUALPROVISIONSRESPONSE,
  '__module__' : 'cosmos.mint.v1beta1.query_pb2'
  # @@protoc_insertion_point(class_scope:cosmos.mint.v1beta1.QueryAnnualProvisionsResponse)
  })
_sym_db.RegisterMessage(QueryAnnualProvisionsResponse)


DESCRIPTOR._options = None
_QUERYPARAMSRESPONSE.fields_by_name['params']._options = None
_QUERYINFLATIONRESPONSE.fields_by_name['inflation']._options = None
_QUERYANNUALPROVISIONSRESPONSE.fields_by_name['annual_provisions']._options = None

_QUERY = _descriptor.ServiceDescriptor(
  name='Query',
  full_name='cosmos.mint.v1beta1.Query',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_start=495,
  serialized_end=948,
  methods=[
  _descriptor.MethodDescriptor(
    name='Params',
    full_name='cosmos.mint.v1beta1.Query.Params',
    index=0,
    containing_service=None,
    input_type=_QUERYPARAMSREQUEST,
    output_type=_QUERYPARAMSRESPONSE,
    serialized_options=b'\202\323\344\223\002\035\022\033/cosmos/mint/v1beta1/params',
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='Inflation',
    full_name='cosmos.mint.v1beta1.Query.Inflation',
    index=1,
    containing_service=None,
    input_type=_QUERYINFLATIONREQUEST,
    output_type=_QUERYINFLATIONRESPONSE,
    serialized_options=b'\202\323\344\223\002 \022\036/cosmos/mint/v1beta1/inflation',
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='AnnualProvisions',
    full_name='cosmos.mint.v1beta1.Query.AnnualProvisions',
    index=2,
    containing_service=None,
    input_type=_QUERYANNUALPROVISIONSREQUEST,
    output_type=_QUERYANNUALPROVISIONSRESPONSE,
    serialized_options=b'\202\323\344\223\002(\022&/cosmos/mint/v1beta1/annual_provisions',
    create_key=_descriptor._internal_create_key,
  ),
])
_sym_db.RegisterServiceDescriptor(_QUERY)

DESCRIPTOR.services_by_name['Query'] = _QUERY

# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2018-2021 Fetch.AI Limited
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------
import random

from cosmpy.aerial.client import LedgerClient
from cosmpy.aerial.config import NetworkConfig
from cosmpy.aerial.tx_helpers import SubmittedTx
from cosmpy.aerial.wallet import LocalWallet
from cosmpy.crypto.keypairs import PrivateKey


def _wait_for_tx(operation: str, tx: SubmittedTx):
    print(f"Waiting for {operation} to complete... (tx: {tx.tx_hash})")
    tx.wait_to_complete()
    print(f"Waiting for {operation} to complete... done")


def main():
    alice = LocalWallet(PrivateKey("p0h0sYImB4xGq3Zz+xfIrY4QR6CPqeNg8w6X3NUWLe4="))

    ledger = LedgerClient(NetworkConfig.fetchai_stable_testnet())

    # get all the active validators on the network
    validators = ledger.query_validators()

    # randomly choose one to stake to (this is a BAD idea generally, but fine for this example)
    first_validator = random.choice(validators)  # nosec

    # delegate some tokens to this validator
    tx = ledger.delegate_tokens(first_validator.address, 20, alice)
    _wait_for_tx("delegate", tx)

    # select another validator to redelegate to (again, not a good idea in general)
    second_validator = random.choice(  # nosec
        list(
            filter(
                lambda v: v.address != first_validator.address,
                validators,
            )
        )
    )

    # redelegate the tokens
    tx = ledger.redelegate_tokens(
        first_validator.address, second_validator.address, 10, alice
    )
    _wait_for_tx("redelegate", tx)

    # undelegate the tokens
    tx = ledger.undelegate_tokens(first_validator.address, 5, alice)
    _wait_for_tx("undelegate", tx)

    summary = ledger.query_staking_summary(alice.address())
    print(
        f"Summary: Staked: {summary.total_staked} Unbonding: {summary.total_unbonding} Rewards: {summary.total_rewards}"
    )

    # finally, lets collect up all the rewards we have earned so far
    claimed = False
    for position in summary.current_positions:
        if position.reward > 0:
            claimed = True

            tx = ledger.claim_rewards(position.validator, alice)
            _wait_for_tx(f"claim from {str(position.validator)}", tx)

    if claimed:
        summary = ledger.query_staking_summary(alice.address())
        print(
            f"Summary: Staked: {summary.total_staked} Unbonding: {summary.total_unbonding} Rewards: {summary.total_rewards}"
        )


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2018-2021 Fetch.AI Limited
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------

"""Implementation of REST api client."""

from typing import List, Optional
from urllib.parse import urlencode

import requests
from google.protobuf.json_format import MessageToDict
from google.protobuf.message import Message


class RestClient:
    """REST api client."""

    def __init__(self, rest_address: str):
        """
        Create REST api client

        :param rest_address: Address of REST node
        """
        self._session = requests.session()
        self.rest_address = rest_address

    def get(
        self,
        url_base_path: str,
        request: Optional[Message] = None,
        used_params: Optional[List[str]] = None,
    ) -> bytes:
        """
        Send a GET request

        :param url_base_path: URL base path
        :param request: Protobuf coded request
        :param used_params: Parameters to be removed from request after converting it to dict

        :raises RuntimeError: if response code is not 200

        :return: Content of response
        """
        if request is None:
            url = f"{self.rest_address}{url_base_path}"
        else:
            json_request = MessageToDict(request)

            # Remove params that are already in url_base_path
            if used_params is not None:
                for param in used_params:
                    json_request.pop(param)

            url_encoded_request = self._url_encode(json_request)

            if len(url_encoded_request) == 0:
                url = f"{self.rest_address}{url_base_path}"
            else:
                url = f"{self.rest_address}{url_base_path}?{url_encoded_request}"

        response = self._session.get(url=url)
        if response.status_code != 200:
            raise RuntimeError(
                f"Error when sending a GET request.\n Response: {response.status_code}, {str(response.content)})"
            )
        return response.content

    def post(self, url_base_path: str, request: Message) -> bytes:
        """
        Send a POST request

        :param url_base_path: URL base path
        :param request: Protobuf coded request

        :raises RuntimeError: if response code is not 200

        :return: Content of response
        """
        json_request = MessageToDict(request)

        headers = {"Content-type": "application/json", "Accept": "application/json"}
        response = self._session.post(
            url=f"{self.rest_address}{url_base_path}",
            json=json_request,
            headers=headers,
        )

        if response.status_code != 200:
            raise RuntimeError(
                f"Error when sending a POST request.\n Request: {json_request}\n Response: {response.status_code}, {str(response.content)})"
            )
        return response.content

    @staticmethod
    def _url_encode(json_request):
        """
        Custom URL encode that breaks down nested dictionaries to match REST api format

        It converts dicts from:
        {"pagination": {"limit": "1", "something": "2"},}

        To:
        {"pagination.limit": "1","pagination.something": "2"}


        :param json_request: JSON request

        :return: urlencoded json_request
        """

        for outer_k, outer_v in json_request.copy().items():
            if isinstance(outer_v, dict):
                for inner_k, inner_v in outer_v.items():
                    json_request[f"{outer_k}.{inner_k}"] = inner_v
                json_request.pop(outer_k)

        return urlencode(json_request, doseq=True)

    def __del__(self):
        self._session.close()



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2018-2022 Fetch.AI Limited
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------

"""BLS Crypto KeyPairs (Public Key and Private Key) and utility functions."""
import base64
import hashlib
from typing import Callable, List, Optional

from blspy import (  # type: ignore  # pylint: disable=no-name-in-module
    AugSchemeMPL,
    G1Element,
    G2Element,
)
from blspy import (
    PrivateKey as BLSPrivateKey,  # pylint: disable=no-name-in-module; type: ignore
)
from ecdsa.curves import NIST256p
from ecdsa.keys import SigningKey

from cosmpy.crypto.interface import Signer


class PublicKey:
    """BLS public key class."""

    HASH_FUNCTION: Callable = hashlib.sha256

    def __init__(self, public_key: bytes):
        """
        Initialize.

        :param public_key: bytes.
        """
        self._public_key_bytes = public_key
        self._public_key = base64.b64encode(self._public_key_bytes).decode()
        self._verifying_key = G1Element.from_bytes(self._public_key_bytes)

    @property
    def public_key(self) -> str:
        """
        Get public key.

        :return: str public key.
        """
        return self._public_key

    @property
    def public_key_hex(self) -> str:
        """
        Get public key hex.

        :return: str public key hex.
        """
        return self.public_key_bytes.hex()

    @property
    def public_key_bytes(self) -> bytes:
        """
        Get bytes public key.

        :return: bytes public key.
        """
        return self._public_key_bytes

    def verify(self, message: bytes, signature: bytes) -> bool:
        """
        Verify message and signature.

        :param message: bytes message content.
        :param signature: bytes signature.
        :return: bool is message and signature valid.
        """
        digest = self.HASH_FUNCTION(message).digest()
        return self.verify_digest(digest, signature)

    def verify_digest(self, digest: bytes, signature: bytes) -> bool:
        """
        Verify digest.

        :param digest: bytes digest.
        :param signature: bytes signature.
        :return: bool is digest valid.
        """
        return AugSchemeMPL.verify(
            self._verifying_key, digest, G2Element.from_bytes(signature)
        )


class PrivateKey(Signer, PublicKey):
    """BLS private key class."""

    HASH_FUNCTION: Callable = hashlib.sha256

    def __init__(self, private_key: Optional[bytes] = None):
        self._private_key_bytes = private_key or self._generate_bytes()
        self._private_key = base64.b64encode(self._private_key_bytes).decode()
        self._signing_key: BLSPrivateKey = AugSchemeMPL.key_gen(self._private_key_bytes)
        PublicKey.__init__(self, public_key=bytes(self._signing_key.get_g1()))

    @property
    def private_key(self) -> str:
        """
        Get private key.

        :return: str private key.
        """
        return self._private_key

    @property
    def private_key_hex(self) -> str:
        """
        Get private key hex.

        :return: str private key hex.
        """
        return self.private_key_bytes.hex()

    @property
    def private_key_bytes(self) -> bytes:
        """
        Get bytes private key.

        :return: bytes private key.
        """
        return self._private_key_bytes

    @staticmethod
    def _generate_bytes() -> bytes:
        """
        Generate random bytes sequence 32 bytes long.

        :return: bytes
        """
        return SigningKey.generate(curve=NIST256p).to_string()

    def sign(
        self, message: bytes, deterministic: bool = True, canonicalise: bool = True
    ) -> bytes:
        """
        Sign message.

        :param message: bytes message content.
        :param deterministic: bool is deterministic.
        :param canonicalise: bool is canonicalise.

        :return: bytes signed message.
        """
        digest = self.HASH_FUNCTION(message).digest()
        return self.sign_digest(digest)

    def sign_digest(
        self, digest: bytes, deterministic=True, canonicalise: bool = True
    ) -> bytes:
        """
        Sign digest.

        :param digest: bytes digest content.
        :param deterministic: bool is deterministic.
        :param canonicalise: bool is canonicalise.

        :return: bytes signed digest.
        """
        return bytes(AugSchemeMPL.sign(self._signing_key, digest))


def aggregate_signatures(*sigs: List[bytes]) -> bytes:
    """
    Combine signatures into one.

    :param *sigs: list of signatures bytes.
    :return: bytes
    """
    return bytes(AugSchemeMPL.aggregate([G2Element.from_bytes(i) for i in sigs]))


def verify_aggregated_signatures(
    pks: List[PublicKey],
    msgs: List[bytes],
    aggregated_signature: bytes,
    hashfunc=hashlib.sha256,
):
    """
    Verify signatures with pub keys and  messages.

    :param pks: list of public keys
    :param msgs: list of messages
    :param aggregated_signature: aggregated signature bytes
    :param hashfunc: hash method from hashlib. default is hashlib.sha256
    :return: bool
    """
    return verify_aggregated_signatures_digests(
        pks, [hashfunc(i).digest() for i in msgs], aggregated_signature
    )


def verify_aggregated_signatures_digests(
    pks: List[PublicKey], digests: List[bytes], aggregated_signature: bytes
):
    """
    Verify signatures with pub keys and  messages.

    :param pks: list of public keys
    :param digests: list of digests calculated
    :param aggregated_signature: aggregated signature bytes
    :return: bool
    """
    return AugSchemeMPL.aggregate_verify(
        [G1Element.from_bytes(pk.public_key_bytes) for pk in pks],
        digests,
        G2Element.from_bytes(aggregated_signature),
    )



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2018-2022 Fetch.AI Limited
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------
"""Interface for the IBC Core Channel functionality of CosmosSDK."""

from abc import ABC, abstractmethod

from cosmpy.protos.ibc.core.channel.v1.query_pb2 import (
    QueryChannelClientStateRequest,
    QueryChannelClientStateResponse,
    QueryChannelConsensusStateRequest,
    QueryChannelConsensusStateResponse,
    QueryChannelRequest,
    QueryChannelResponse,
    QueryChannelsRequest,
    QueryChannelsResponse,
    QueryConnectionChannelsRequest,
    QueryConnectionChannelsResponse,
    QueryNextSequenceReceiveRequest,
    QueryNextSequenceReceiveResponse,
    QueryPacketAcknowledgementRequest,
    QueryPacketAcknowledgementResponse,
    QueryPacketAcknowledgementsRequest,
    QueryPacketAcknowledgementsResponse,
    QueryPacketCommitmentRequest,
    QueryPacketCommitmentResponse,
    QueryPacketCommitmentsRequest,
    QueryPacketCommitmentsResponse,
    QueryPacketReceiptRequest,
    QueryPacketReceiptResponse,
    QueryUnreceivedAcksRequest,
    QueryUnreceivedAcksResponse,
    QueryUnreceivedPacketsRequest,
    QueryUnreceivedPacketsResponse,
)


class IBCCoreChannel(ABC):
    """IBC Core Channel abstract class."""

    @abstractmethod
    def Channel(self, request: QueryChannelRequest) -> QueryChannelResponse:
        """
        Channel queries an IBC Channel.

        :param request: QueryChannelRequest
        :return: QueryChannelResponse
        """

    @abstractmethod
    def Channels(self, request: QueryChannelsRequest) -> QueryChannelsResponse:
        """
        Channels queries all the IBC channels of a chain.

        :param request: QueryChannelsRequest
        :return: QueryChannelsResponse
        """

    @abstractmethod
    def ConnectionChannels(
        self, request: QueryConnectionChannelsRequest
    ) -> QueryConnectionChannelsResponse:
        """
        ConnectionChannels queries all the channels associated with a connection.

        :param request: QueryConnectionChannelsRequest
        :return: QueryConnectionChannelsResponse
        """

    @abstractmethod
    def ChannelClientState(
        self, request: QueryChannelClientStateRequest
    ) -> QueryChannelClientStateResponse:
        """
        ChannelClientState queries for the client state for the channel associated with the provided channel identifiers.

        :param request: QueryChannelClientStateRequest
        :return: QueryChannelClientStateResponse
        """

    @abstractmethod
    def ChannelConsensusState(
        self, request: QueryChannelConsensusStateRequest
    ) -> QueryChannelConsensusStateResponse:
        """
        ChannelConsensusState queries for the consensus state for the channel associated with the provided channel identifiers.

        :param request: QueryChannelConsensusStateRequest
        :return: QueryChannelConsensusStateResponse
        """

    @abstractmethod
    def PacketCommitment(
        self, request: QueryPacketCommitmentRequest
    ) -> QueryPacketCommitmentResponse:
        """
        PacketCommitment queries a stored packet commitment hash.

        :param request: QueryPacketCommitmentRequest
        :return: QueryPacketCommitmentResponse
        """

    @abstractmethod
    def PacketCommitments(
        self, request: QueryPacketCommitmentsRequest
    ) -> QueryPacketCommitmentsResponse:
        """
        PacketCommitments returns all the packet commitments hashes associated with a channel.

        :param request: QueryPacketCommitmentsRequest
        :return: QueryPacketCommitmentsResponse
        """

    @abstractmethod
    def PacketReceipt(
        self, request: QueryPacketReceiptRequest
    ) -> QueryPacketReceiptResponse:
        """
        PacketReceipt queries if a given packet sequence has been received on the queried chain

        :param request: QueryPacketReceiptRequest
        :return: QueryPacketReceiptResponse
        """

    @abstractmethod
    def PacketAcknowledgement(
        self, request: QueryPacketAcknowledgementRequest
    ) -> QueryPacketAcknowledgementResponse:
        """
        PacketAcknowledgement queries a stored packet acknowledgment hash.

        :param request: QueryPacketAcknowledgementRequest
        :return: QueryPacketAcknowledgementResponse
        """

    @abstractmethod
    def PacketAcknowledgements(
        self, request: QueryPacketAcknowledgementsRequest
    ) -> QueryPacketAcknowledgementsResponse:
        """
        PacketAcknowledgements returns all the packet acknowledgments associated with a channel.

        :param request: QueryPacketAcknowledgementsRequest
        :return: QueryPacketAcknowledgementsResponse
        """

    @abstractmethod
    def UnreceivedPackets(
        self, request: QueryUnreceivedPacketsRequest
    ) -> QueryUnreceivedPacketsResponse:
        """
        UnreceivedPackets returns all the unreceived IBC packets associated with a channel and sequences.

        :param request: QueryUnreceivedPacketsRequest
        :return: QueryUnreceivedPacketsResponse
        """

    @abstractmethod
    def UnreceivedAcks(
        self, request: QueryUnreceivedAcksRequest
    ) -> QueryUnreceivedAcksResponse:
        """
        UnreceivedAcks returns all the unreceived IBC acknowledgments associated with a channel and sequences.

        :param request: QueryUnreceivedAcksRequest
        :return: QueryUnreceivedAcksResponse
        """

    @abstractmethod
    def NextSequenceReceive(
        self, request: QueryNextSequenceReceiveRequest
    ) -> QueryNextSequenceReceiveResponse:
        """
        NextSequenceReceive returns the next receive sequence for a given channel.

        :param request: QueryNextSequenceReceiveRequest
        :return: QueryNextSequenceReceiveResponse
        """



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2018-2022 Fetch.AI Limited
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------
"""Tests for REST implementation of IBC Core Channel."""
import json
from typing import Dict, Tuple
from unittest import TestCase

from google.protobuf.json_format import ParseDict
from google.protobuf.wrappers_pb2 import Int32Value  # noqa # needed for protobuf decode

from cosmpy.ibc.core.channel.rest_client import IBCCoreChannelRestClient  # type: ignore
from cosmpy.protos.ibc.core.channel.v1.query_pb2 import (
    QueryChannelClientStateRequest,
    QueryChannelClientStateResponse,
    QueryChannelConsensusStateRequest,
    QueryChannelConsensusStateResponse,
    QueryChannelRequest,
    QueryChannelResponse,
    QueryChannelsRequest,
    QueryChannelsResponse,
    QueryConnectionChannelsRequest,
    QueryConnectionChannelsResponse,
    QueryNextSequenceReceiveRequest,
    QueryNextSequenceReceiveResponse,
    QueryPacketAcknowledgementRequest,
    QueryPacketAcknowledgementResponse,
    QueryPacketAcknowledgementsRequest,
    QueryPacketAcknowledgementsResponse,
    QueryPacketCommitmentRequest,
    QueryPacketCommitmentResponse,
    QueryPacketCommitmentsRequest,
    QueryPacketCommitmentsResponse,
    QueryPacketReceiptRequest,
    QueryPacketReceiptResponse,
    QueryUnreceivedAcksRequest,
    QueryUnreceivedAcksResponse,
    QueryUnreceivedPacketsRequest,
    QueryUnreceivedPacketsResponse,
)
from tests.helpers import MockRestClient

TYPE = {
    "@type": "type.googleapis.com/google.protobuf.Int32Value",
    "value": "42",
}


class IBCCoreChannelRestClientTestCase(TestCase):
    """Test case for IBCCoreChannelRestClient class."""

    REST_CLIENT = IBCCoreChannelRestClient

    def make_clients(
        self, response_content: Dict
    ) -> Tuple[MockRestClient, IBCCoreChannelRestClient]:
        """
        Make  mock client and rest client api for specific content.

        :param response_content: dict
        :return: rest client instance
        """
        mock_client = MockRestClient(json.dumps(response_content).encode("utf-8"))
        rest_client = self.REST_CLIENT(mock_client)
        return mock_client, rest_client

    def test_Channel(self):
        """Test Channel method."""
        content = {
            "channel": {
                "state": "STATE_UNINITIALIZED_UNSPECIFIED",
                "ordering": "ORDER_NONE_UNSPECIFIED",
                "counterparty": {"port_id": "1", "channel_id": "1"},
                "connection_hops": ["1"],
                "version": "1",
            },
            "proof": "string",
            "proof_height": {"revision_number": "1", "revision_height": "1"},
        }
        mock_client, rest_client = self.make_clients(content)
        expected_response = ParseDict(content, QueryChannelResponse())

        assert (
            rest_client.Channel(
                QueryChannelRequest(channel_id="channel_id", port_id="port_id")
            )
            == expected_response
        )
        assert (
            mock_client.last_base_url
            == "/ibc/core/channel/v1beta1/channels/channel_id/ports/port_id"
        )

    def test_Channels(self):
        """Test Channels method."""
        content = {
            "channels": [
                {
                    "state": "STATE_UNINITIALIZED_UNSPECIFIED",
                    "ordering": "ORDER_NONE_UNSPECIFIED",
                    "counterparty": {"port_id": "1", "channel_id": "1"},
                    "connection_hops": ["1"],
                    "version": "1",
                    "port_id": "1",
                    "channel_id": "1",
                }
            ],
            "pagination": {"next_key": "string", "total": "1"},
            "height": {"revision_number": "1", "revision_height": "1"},
        }
        mock_client, rest_client = self.make_clients(content)
        expected_response = ParseDict(content, QueryChannelsResponse())

        assert rest_client.Channels(QueryChannelsRequest()) == expected_response
        assert mock_client.last_base_url == "/ibc/core/channel/v1beta1/channels"

    def test_ConnectionChannels(self):
        """Test ConnectionChannels method."""
        content = {
            "channels": [
                {
                    "state": "STATE_UNINITIALIZED_UNSPECIFIED",
                    "ordering": "ORDER_NONE_UNSPECIFIED",
                    "counterparty": {"port_id": "1", "channel_id": "1"},
                    "connection_hops": ["1"],
                    "version": "1",
                    "port_id": "1",
                    "channel_id": "1",
                }
            ],
            "pagination": {"next_key": "string", "total": "1"},
            "height": {"revision_number": "1", "revision_height": "1"},
        }
        mock_client, rest_client = self.make_clients(content)
        expected_response = ParseDict(content, QueryConnectionChannelsResponse())

        assert (
            rest_client.ConnectionChannels(
                QueryConnectionChannelsRequest(connection="connection")
            )
            == expected_response
        )
        assert (
            mock_client.last_base_url
            == "/ibc/core/channel/v1beta1/connections/connection/channels"
        )

    def test_ChannelClientState(self):
        """Test ChannelClientState method."""
        content = {
            "identified_client_state": {"client_id": "1", "client_state": TYPE},
            "proof": "string",
            "proof_height": {"revision_number": "1", "revision_height": "1"},
        }
        mock_client, rest_client = self.make_clients(content)
        expected_response = ParseDict(content, QueryChannelClientStateResponse())

        assert (
            rest_client.ChannelClientState(
                QueryChannelClientStateRequest(channel_id="1", port_id="1")
            )
            == expected_response
        )
        assert (
            mock_client.last_base_url
            == "/ibc/core/channel/v1beta1/channels/1/ports/1/client_state"
        )

    def test_ChannelConsensusState(self):
        """Test ChannelConsensusState method."""
        content = {
            "consensus_state": TYPE,
            "client_id": "1",
            "proof": "string",
            "proof_height": {
                "revision_number": "1",
            },
        }
        mock_client, rest_client = self.make_clients(content)
        expected_response = ParseDict(content, QueryChannelConsensusStateResponse())

        assert (
            rest_client.ChannelConsensusState(
                QueryChannelConsensusStateRequest(
                    channel_id="1", port_id="1", revision_number=1, revision_height=1
                )
            )
            == expected_response
        )
        assert (
            mock_client.last_base_url
            == "/ibc/core/channel/v1beta1/channels/1/ports/1/consensus_state/revision/1/height/1"
        )

    def test_PacketCommitment(self):
        """Test PacketCommitment method."""
        content = {
            "commitment": "string",
            "proof": "string",
            "proof_height": {"revision_number": "1", "revision_height": "1"},
        }
        mock_client, rest_client = self.make_clients(content)
        expected_response = ParseDict(content, QueryPacketCommitmentResponse())

        assert (
            rest_client.PacketCommitment(
                QueryPacketCommitmentRequest(sequence=1, channel_id="1", port_id="1")
            )
            == expected_response
        )
        assert (
            mock_client.last_base_url
            == "/ibc/core/channel/v1beta1/channels/1/ports/1/packet_commitments/1"
        )

    def test_PacketCommitments(self):
        """Test PacketCommitments method."""
        content = {
            "commitments": [
                {
                    "port_id": "1",
                    "channel_id": "1",
                    "sequence": "1",
                    "data": "string",
                }
            ],
            "pagination": {"next_key": "string", "total": "1"},
            "height": {"revision_number": "1", "revision_height": "1"},
        }
        mock_client, rest_client = self.make_clients(content)
        expected_response = ParseDict(content, QueryPacketCommitmentsResponse())

        assert (
            rest_client.PacketCommitments(
                QueryPacketCommitmentsRequest(channel_id="1", port_id="1")
            )
            == expected_response
        )
        assert (
            mock_client.last_base_url
            == "/ibc/core/channel/v1beta1/channels/1/ports/1/packet_commitments"
        )

    def test_PacketReceipt(self):
        """Test PacketReceipt method."""
        content = {
            "received": True,
            "proof": "string",
            "proof_height": {"revision_number": "1", "revision_height": "1"},
        }
        mock_client, rest_client = self.make_clients(content)
        expected_response = ParseDict(content, QueryPacketReceiptResponse())

        assert (
            rest_client.PacketReceipt(
                QueryPacketReceiptRequest(channel_id="1", port_id="1", sequence=1)
            )
            == expected_response
        )
        assert (
            mock_client.last_base_url
            == "/ibc/core/channel/v1beta1/channels/1/ports/1/packet_receipts/1"
        )

    def test_PacketAcknowledgement(self):
        """Test PacketAcknowledgement method."""
        content = {
            "acknowledgement": "string",
            "proof": "string",
            "proof_height": {"revision_number": "1", "revision_height": "1"},
        }
        mock_client, rest_client = self.make_clients(content)
        expected_response = ParseDict(content, QueryPacketAcknowledgementResponse())

        assert (
            rest_client.PacketAcknowledgement(
                QueryPacketAcknowledgementRequest(
                    port_id="1", channel_id="1", sequence=1
                )
            )
            == expected_response
        )
        assert (
            mock_client.last_base_url
            == "/ibc/core/channel/v1beta1/channels/1/ports/1/packet_acks/1"
        )

    def test_PacketAcknowledgements(self):
        """Test PacketAcknowledgements method."""
        content = {
            "acknowledgements": [
                {"port_id": "1", "channel_id": "1", "sequence": "1", "data": "string"}
            ],
            "pagination": {"next_key": "string", "total": "1"},
            "height": {"revision_number": "1", "revision_height": "1"},
        }
        mock_client, rest_client = self.make_clients(content)
        expected_response = ParseDict(content, QueryPacketAcknowledgementsResponse())

        assert (
            rest_client.PacketAcknowledgements(
                QueryPacketAcknowledgementsRequest(channel_id="1", port_id="1")
            )
            == expected_response
        )
        assert (
            mock_client.last_base_url
            == "/ibc/core/channel/v1beta1/channels/1/ports/1/packet_acknowledgements"
        )

    def test_UnreceivedPackets(self):
        """Test UnreceivedPackets method."""
        content = {
            "sequences": ["1"],
            "height": {"revision_number": "1", "revision_height": "1"},
        }
        mock_client, rest_client = self.make_clients(content)
        expected_response = ParseDict(content, QueryUnreceivedPacketsResponse())

        assert (
            rest_client.UnreceivedPackets(
                QueryUnreceivedPacketsRequest(
                    channel_id="1", port_id="1", packet_commitment_sequences=[1]
                )
            )
            == expected_response
        )
        assert (
            mock_client.last_base_url
            == "/ibc/core/channel/v1beta1/channels/1/ports/1/packet_commitments/1/unreceived_packets"
        )

    def test_UnreceivedAcks(self):
        """Test UnreceivedAcks method."""
        content = {
            "sequences": ["1"],
            "height": {"revision_number": "1", "revision_height": "1"},
        }
        mock_client, rest_client = self.make_clients(content)
        expected_response = ParseDict(content, QueryUnreceivedAcksResponse())

        assert (
            rest_client.UnreceivedAcks(
                QueryUnreceivedAcksRequest(
                    packet_ack_sequences=[1], channel_id="1", port_id="1"
                )
            )
            == expected_response
        )
        assert (
            mock_client.last_base_url
            == "/ibc/core/channel/v1beta1/channels/1/ports/1/packet_commitments/1/unreceived_acks"
        )

    def test_NextSequenceReceive(self):
        """Test NextSequenceReceive method."""
        content = {
            "next_sequence_receive": "1",
            "proof": "string",
            "proof_height": {"revision_number": "1", "revision_height": "1"},
        }
        mock_client, rest_client = self.make_clients(content)
        expected_response = ParseDict(content, QueryNextSequenceReceiveResponse())

        assert (
            rest_client.NextSequenceReceive(
                QueryNextSequenceReceiveRequest(port_id="1", channel_id="1")
            )
            == expected_response
        )
        assert (
            mock_client.last_base_url
            == "/ibc/core/channel/v1beta1/channels/1/ports/1/next_sequence"
        )
